/* (C) Copyright 1984,85,86,87 Walter L. Peacock   All Rights Reserved    */
/* B T I N S P L T . C     7/24/84   */

/*---------------------------------------------------------------------------- 
 * insertky() is used to insert a new key into the B+tree index page.
 *---------------------------------------------------------------------------*/

#include "cbtree.h"

/* this function must NOT be called unless there is REALLY enough room */
/* in the btpage to hold the new key */

void insertky(pblk, hdrloc, srchkey, keylen)  /* insert key into btpage */

BTIDXBLK *pblk;
int hdrloc;
char *srchkey;
unsigned keylen;
{
   extern char *calloc(), *strsave();
   char *cp;

   /* if necessary, get memory for new key */
   if (pblk->blkalloc <= pblk->cellicnt)
   {
      cp = strsave(srchkey, keylen);
      pblk->blkalloc++;
   }
   else
   {
      cp = pblk->btpage[pblk->cellicnt].skeynme;     /* re-use top one */
      strnncpy(cp, srchkey, keylen);
   }

   /* open up a slot */
   blkshft(pblk->btpage + hdrloc + 1, pblk->btpage + hdrloc,
         pblk->cellicnt - hdrloc, SZBTIDX);
   ++pblk->cellicnt;    /* adjust cell count */

   pblk->btpage[hdrloc].skeynme = cp;    /* new one goes here */
}


/*   spltnode: function to split btree node       */
/*---------------------------------------------------------------------------- 
 * spltnode() is called when a key insertion will result in the index page 
 * containing more than the max cells specified during BTSETUP.EXE.   The
 * index node will be split into two pages of equal size +/- 1.
 *---------------------------------------------------------------------------*/

void spltnode(pblk, nublk, splitpt)
BTIDXBLK *pblk;
BTIDXBLK *nublk;     /* this better be an empty index block */
unsigned  splitpt;
{
   uword count;
   uword u;

   count = pblk->cellicnt - splitpt;   /* # blocks to move */

   /* free up space in nublk for copy */
   u = MIN(nublk->blkalloc, count);    /* # to free up in nublk */
   nublk->blkalloc -= u;      /* reduce allocation count */
   while (u--)
      FREE(nublk->btpage[u].skeynme);

   /* copy 'count' blocks from pblk->btpage to nublk->btpage */
   blkshft(nublk->btpage, pblk->btpage + splitpt, count, SZBTIDX);
   nublk->blkalloc += count;
   nublk->cellicnt  = count;
   pblk->blkalloc  -= count;
   pblk->cellicnt  -= count;

   /* collapse excess stuff in pblk over stuff just moved */
   if ((u = pblk->blkalloc - pblk->cellicnt) != 0)    /* # excess blocks */
      blkshft(pblk->btpage + splitpt, pblk->btpage + splitpt + count, u,
            SZBTIDX);
}
